﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace SelectedRowItem
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            vm.SelectedProducts.CollectionChanged += SelectedProducts_CollectionChanged;
        }

        private void SelectedProducts_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            foreach(var item in vm.SelectedProducts)
            {
                var row = dataGrid.Rows.FirstOrDefault(r => r.DataItem == item);
                dataGrid.Selection.Add(row, row);
            }
        }

        private void DataGrid_LoadedCellPresenter(object sender, C1.WPF.DataGrid.DataGridCellEventArgs e)
        {
            if(e.Cell.Column.Name == "IsSelected")
            {
                (e.Cell.Presenter.Content as CheckBox).Tag = e.Cell;
                (e.Cell.Presenter.Content as CheckBox).Checked += MainWindow_Checked;
                (e.Cell.Presenter.Content as CheckBox).Unchecked += MainWindow_Unchecked;
            }
        }

        private void MainWindow_Unchecked(object sender, RoutedEventArgs e)
        {
            C1.WPF.DataGrid.DataGridCell cell = (sender as CheckBox).Tag as C1.WPF.DataGrid.DataGridCell;
            dataGrid.Selection.Remove(cell.Row, cell.Row);
            vm.SelectedProducts.Remove(cell.Row.DataItem as Product);
        }

        private void MainWindow_Checked(object sender, RoutedEventArgs e)
        {
            C1.WPF.DataGrid.DataGridCell cell = (sender as CheckBox).Tag as C1.WPF.DataGrid.DataGridCell;
            vm.SelectedProducts.Add(cell.Row.DataItem as Product);
        }
        private void DataGrid_UnloadedCellPresenter(object sender, C1.WPF.DataGrid.DataGridCellEventArgs e)
        {
            if (e.Cell.Column.Name == "IsSelected")
            {
                (e.Cell.Presenter.Content as CheckBox).Checked -= MainWindow_Checked;
                (e.Cell.Presenter.Content as CheckBox).Unchecked -= MainWindow_Unchecked;
            }
        }
    }
    public class ViewModel
    {
        private ObservableCollection<Product> _products;
        private ObservableCollection<Product> _selectedProducts;
        public ObservableCollection<Product> Products
        {
            get
            {
                if (_products == null)
                    InitCustomProducts();
                return _products;
            }
        }

        public ObservableCollection<Product> SelectedProducts
        {
            get
            {
                if (_selectedProducts == null)
                    _selectedProducts = new ObservableCollection<Product>();
                return _selectedProducts;
            }
        }
        private void InitCustomProducts()
        {
            _products = new ObservableCollection<Product>();
            for (int i = 0; i < 100; i++)
            {
                _products.Add(new Product());
            }
        }
        public ICommand DisplaySelectedItemCommand
        {
            get;set;
        }
        public ViewModel()
        {
            DisplaySelectedItemCommand = new RelayCommand(ExecuteMyMethod, CanExecuteMyMethod);
        }
        private bool CanExecuteMyMethod(object parameter)
        {
            if (parameter != null && ((object[])parameter).Count() > 0) //Or directly use SelectedProducts instead of parameter.
                return true;
            else
                return false;
        }

        private void ExecuteMyMethod(object parameter)
        {
            var selectedRowItems = ((object [])parameter).Cast<Product>(); //Or directly use SelectedProducts instead of parameter.

            string info = "SelectedItem Info: \n";
            foreach (var item in selectedRowItems)
            {
                info += $"{item.Name}, {item.Color}, {item.Line}, {item.Price}\n";
            }

            MessageBox.Show(info);
        }
    }
    public class Product
    {
        static Random _rnd = new Random();
        static string[] _names = "Macko|Surfair|Pocohey|Studeby".Split('|');
        static string[] _lines = "Computers|Washers|Stoves|Cars".Split('|');
        static string[] _colors = "Red|Green|Blue|White".Split('|');
        static string[] _priorities = "Low|Medium|High".Split('|');

        public bool IsSelected { get; set; }
        public string Name { get; set; }
        public string Color { get; set; }
        public string Line { get; set; }
        public double Price { get; set; }
        public double Cost { get; set; }
        public DateTime Introduced { get; set; }
        public int Priority { get; set; }
        public Product()
        {
            Name = _names[_rnd.Next() % _names.Length];
            Line = _lines[_rnd.Next() % _lines.Length];
            Color = _colors[_rnd.Next() % _colors.Length];
            Price = 30 + _rnd.NextDouble() * 1000;
            Cost = 3 + _rnd.NextDouble() * 300;
            Introduced = DateTime.Today.AddDays(_rnd.Next(-600, 0));
            Priority = _rnd.Next(0, 3);
        }
    }
    public class RelayCommand : ICommand
    {
        Action<object> _executeMethod;
        Func<object, bool> _canexecuteMethod;


        public RelayCommand(Action<object> executeMethod, Func<object, bool> canexecuteMethod)
        {
            _executeMethod = executeMethod;
            _canexecuteMethod = canexecuteMethod;
        }

        public bool CanExecute(object parameter)
        {
            if (_canexecuteMethod != null)
            {
                return _canexecuteMethod(parameter);
            }
            else
            {
                return false;
            }
        }

        public event EventHandler CanExecuteChanged {
            add
            {
                CommandManager.RequerySuggested += value;
            }
            remove
            {
                CommandManager.RequerySuggested -= value;
            }
        }

        public void Execute(object parameter)
        {
            _executeMethod(parameter);
        }
    }
}
